<?php
/**
 * @copyright	Copyright (C) 2011 Simplify Your Web, Inc. All rights reserved.
 * @license		GNU General Public License version 3 or later; see LICENSE.txt
 */

// No direct access to this file
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

/**
 * Script file for the Latest News Enhanced Pro Newsfeeds plugin
 */
class plglatestnewsenhancednewsfeedsInstallerScript
{
	static $version = '5.1.1';
	static $minimum_needed_lnee_version = '5.0.0';
	static $download_link = 'https://simplifyyourweb.com/downloads/latest-news-enhanced-pro';

	/**
	 * Called before an install/update method
	 *
	 * @return  boolean  True on success
	 */
	public function preflight($type, $parent)
	{
		// check if Latest News Enhanced Extended Pro is installed and of version v2.2.0+

		if (!JFolder::exists(JPATH_ROOT.'/modules/mod_latestnewsenhancedextended')) {

			$message = Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_MISSING_MODULE').'.<br /><a href="'.self::$download_link.'" target="_blank">'.Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_DOWNLOAD_MODULE').'</a>.';
			Factory::getApplication()->enqueueMessage($message, 'error');
			return false;

		} else {
			$module_version = strval(simplexml_load_file(JPATH_SITE . '/modules/mod_latestnewsenhancedextended/mod_latestnewsenhancedextended.xml')->version);

			if (version_compare($module_version, self::$minimum_needed_lnee_version, 'ge')) {
				Factory::getApplication()->enqueueMessage(Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_COMPATIBLE_MODULE'), 'message');
				return true;
			} else {
				$message = Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_NONCOMPATIBLE_MODULE').'.<br /><a href="'.self::$download_link.'" target="_blank">'.Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_UPDATE_MODULE').'</a>.';
				Factory::getApplication()->enqueueMessage($message, 'error');
				return false;
			}
		}
	}

	/**
	 * Called after an install/update method
	 *
	 * @return  boolean  True on success
	 */
	public function postflight($type, $parent)
	{
		echo '<p style="margin: 20px 0">';
		echo Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_VERSION').' <span class="label">'.self::$version.'</span>';
		echo '<br /><br />Olivier Buisard @ <a href="https://simplifyyourweb.com" target="_blank">Simplify Your Web</a>';
		echo '</p>';

		if ($type == 'update') {

			// update warning

			Factory::getApplication()->enqueueMessage(Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_WARNING_RELEASENOTES'), 'warning');
		}

		// move layouts to LNEP extension

		$layoutfiles = array();
		$layoutfiles[] = 'lnep_detail_newsfeeds_feedtitle.php';
		$layoutfiles[] = 'lnep_detail_newsfeeds_entrycategories.php';
		$layoutfiles[] = 'lnep_detail_newsfeeds_entrysource.php';
		$layoutfiles[] = 'lnep_detail_newsfeeds_fields.php';

		foreach ($layoutfiles as $layoutfile) {
		    $src = JPATH_ROOT.'/plugins/latestnewsenhanced/newsfeeds/layouts/'.$layoutfile;
		    $dest = JPATH_ROOT.'/components/com_latestnewsenhancedpro/layouts/details/'.$layoutfile;

		    if (!JFile::copy($src, $dest)) {
		        Factory::getApplication()->enqueueMessage(Text::sprintf('PLG_LATESTNEWSENHANCED_NEWSFEEDS_WARNING_COULDNOTCOPYFILE', $layoutfile), 'warning');
		    }
		}

		// enable the plugin

		$this->enableExtension('plugin', 'newsfeeds', 'latestnewsenhanced');

		// overrides warning

		$defaultemplate = $this->getDefaultTemplate();

		if ($defaultemplate) {
			$overrides_path = JPATH_ROOT.'/templates/'.$defaultemplate.'/html/';

			if (JFolder::exists($overrides_path.'plg_latestnewsenhanced_newsfeeds')) {
				Factory::getApplication()->enqueueMessage(Text::_('PLG_LATESTNEWSENHANCED_NEWSFEEDS_WARNING_OVERRIDES'), 'warning');
			}
		}

		return true;
	}

	private function enableExtension($type, $element, $folder = '')
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true);

		$query->update($db->quoteName('#__extensions'));
		$query->set($db->quoteName('enabled').' = 1');
		$query->where($db->quoteName('type').' = '.$db->quote($type));
		$query->where($db->quoteName('element').' = '.$db->quote($element));
		$query->where($db->quoteName('folder').' = '.$db->quote($folder));

		$db->setQuery($query);

		try {
			$db->execute();
		} catch (\RuntimeException $e) {
			if ($db->getErrorNum()) {
				Factory::getApplication()->enqueueMessage($db->getErrorMsg(), 'warning');
			} else {
				Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
			}
			return false;
		}

		return true;
	}

	private function getDefaultTemplate()
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true);

		$query->select('template');
		$query->from('#__template_styles');
		$query->where($db->quoteName('client_id').'= 0');
		$query->where($db->quoteName('home').'= 1');

		$db->setQuery($query);

		$defaultemplate = '';

		try {
			$defaultemplate = $db->loadResult();
		} catch (\RuntimeException $e) {
			if ($db->getErrorNum()) {
				Factory::getApplication()->enqueueMessage($db->getErrorMsg(), 'warning');
			} else {
				Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
			}
		}

		return $defaultemplate;
	}

	/**
	 * Called on installation
	 *
	 * @return  boolean  True on success
	 */
	public function install($parent) { }

	/**
	 * Called on update
	 *
	 * @return  boolean  True on success
	 */
	public function update($parent) { }

	/**
	 * Called on uninstallation
	 */
	public function uninstall($parent) { }

}
?>